@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     pygments.lexers.asm
     ~~~~~~~~~~~~~~~~~~~
@@ -16,7 +15,7 @@ from pygments.lexer import RegexLexer, include, bygroups, using, words, \
 from pygments.lexers.c_cpp import CppLexer, CLexer
 from pygments.lexers.d import DLexer
 from pygments.token import Text, Name, Number, String, Comment, Punctuation, \
-    Other, Keyword, Operator, Literal
+    Other, Keyword, Operator, Whitespace
 
 __all__ = ['GasLexer', 'ObjdumpLexer', 'DObjdumpLexer', 'CppObjdumpLexer',
            'CObjdumpLexer', 'HsailLexer', 'LlvmLexer', 'LlvmMirBodyLexer',
@@ -38,7 +37,7 @@ class GasLexer(RegexLexer):
     char = r'[\w$.@-]'
     identifier = r'(?:[a-zA-Z$_]' + char + r'*|\.' + char + '+)'
     number = r'(?:0[xX][a-fA-F0-9]+|#?-?\d+)'
-    register = '%' + identifier
+    register = '%' + identifier + r'\b'
 
     tokens = {
         'root': [
@@ -55,7 +54,7 @@ class GasLexer(RegexLexer):
             ('@' + identifier, Name.Attribute),
             (number, Number.Integer),
             (register, Name.Variable),
-            (r'[\r\n]+', Text, '#pop'),
+            (r'[\r\n]+', Whitespace, '#pop'),
             (r'([;#]|//).*?\n', Comment.Single, '#pop'),
             (r'/[*].*?[*]/', Comment.Multiline),
             (r'/[*].*?\n[\w\W]*?[*]/', Comment.Multiline, '#pop'),
@@ -81,7 +80,7 @@ class GasLexer(RegexLexer):
             # Numeric constants
             ('$'+number, Number.Integer),
             (r"$'(.|\\')'", String.Char),
-            (r'[\r\n]+', Text, '#pop'),
+            (r'[\r\n]+', Whitespace, '#pop'),
             (r'([;#]|//).*?\n', Comment.Single, '#pop'),
             (r'/[*].*?[*]/', Comment.Multiline),
             (r'/[*].*?\n[\w\W]*?[*]/', Comment.Multiline, '#pop'),
@@ -90,8 +89,8 @@ class GasLexer(RegexLexer):
             include('whitespace')
         ],
         'whitespace': [
-            (r'\n', Text),
-            (r'\s+', Text),
+            (r'\n', Whitespace),
+            (r'\s+', Whitespace),
             (r'([;#]|//).*?\n', Comment.Single),
             (r'/[*][\w\W]*?[*]/', Comment.Multiline)
         ],
@@ -123,33 +122,37 @@ def _objdump_lexer_tokens(asm_lexer):
             # Function labels
             # (With offset)
             ('('+hex_re+'+)( )(<)(.*?)([-+])(0[xX][A-Za-z0-9]+)(>:)$',
-                bygroups(Number.Hex, Text, Punctuation, Name.Function,
+                bygroups(Number.Hex, Whitespace, Punctuation, Name.Function,
                          Punctuation, Number.Hex, Punctuation)),
             # (Without offset)
             ('('+hex_re+'+)( )(<)(.*?)(>:)$',
-                bygroups(Number.Hex, Text, Punctuation, Name.Function,
+                bygroups(Number.Hex, Whitespace, Punctuation, Name.Function,
                          Punctuation)),
             # Code line with disassembled instructions
             ('( *)('+hex_re+r'+:)(\t)((?:'+hex_re+hex_re+' )+)( *\t)([a-zA-Z].*?)$',
-                bygroups(Text, Name.Label, Text, Number.Hex, Text,
+                bygroups(Whitespace, Name.Label, Whitespace, Number.Hex, Whitespace,
+                         using(asm_lexer))),
+            # Code line without raw instructions (objdump --no-show-raw-insn)
+            ('( *)('+hex_re+r'+:)( *\t)([a-zA-Z].*?)$',
+                bygroups(Whitespace, Name.Label, Whitespace,
                          using(asm_lexer))),
             # Code line with ascii
             ('( *)('+hex_re+r'+:)(\t)((?:'+hex_re+hex_re+' )+)( *)(.*?)$',
-                bygroups(Text, Name.Label, Text, Number.Hex, Text, String)),
+                bygroups(Whitespace, Name.Label, Whitespace, Number.Hex, Whitespace, String)),
             # Continued code line, only raw opcodes without disassembled
             # instruction
             ('( *)('+hex_re+r'+:)(\t)((?:'+hex_re+hex_re+' )+)$',
-                bygroups(Text, Name.Label, Text, Number.Hex)),
+                bygroups(Whitespace, Name.Label, Whitespace, Number.Hex)),
             # Skipped a few bytes
             (r'\t\.\.\.$', Text),
             # Relocation line
             # (With offset)
             (r'(\t\t\t)('+hex_re+r'+:)( )([^\t]+)(\t)(.*?)([-+])(0x'+hex_re+'+)$',
-                bygroups(Text, Name.Label, Text, Name.Property, Text,
+                bygroups(Whitespace, Name.Label, Whitespace, Name.Property, Whitespace,
                          Name.Constant, Punctuation, Number.Hex)),
             # (Without offset)
             (r'(\t\t\t)('+hex_re+r'+:)( )([^\t]+)(\t)(.*?)$',
-                bygroups(Text, Name.Label, Text, Name.Property, Text,
+                bygroups(Whitespace, Name.Label, Whitespace, Name.Property, Whitespace,
                          Name.Constant)),
             (r'[^\n]+\n', Other)
         ]
@@ -222,7 +225,7 @@ class HsailLexer(RegexLexer):
     identifier = r'[a-zA-Z_][\w.]*'
     # Registers
     register_number = r'[0-9]+'
-    register = r'(\$(c|s|d|q)' + register_number + ')'
+    register = r'(\$(c|s|d|q)' + register_number + r')\b'
     # Qualifiers
     alignQual = r'(align\(\d+\))'
     widthQual = r'(width\((\d+|all)\))'
@@ -271,7 +274,7 @@ class HsailLexer(RegexLexer):
             (r'[=<>{}\[\]()*.,:;!]|x\b', Punctuation)
         ],
         'whitespace': [
-            (r'(\n|\s)+', Text),
+            (r'(\n|\s)+', Whitespace),
         ],
         'comments': [
             (r'/\*.*?\*/', Comment.Multiline),
@@ -354,13 +357,14 @@ class LlvmLexer(RegexLexer):
     #: optional Comment or Whitespace
     string = r'"[^"]*?"'
     identifier = r'([-a-zA-Z$._][\w\-$.]*|' + string + ')'
+    block_label = r'(' + identifier + r'|(\d+))'
 
     tokens = {
         'root': [
             include('whitespace'),
 
             # Before keywords, because keywords are valid label names :(...
-            (identifier + r'\s*:', Name.Label),
+            (block_label + r'\s*:', Name.Label),
 
             include('keyword'),
 
@@ -379,74 +383,101 @@ class LlvmLexer(RegexLexer):
             (r'[=<>{}\[\]()*.,!]|x\b', Punctuation)
         ],
         'whitespace': [
-            (r'(\n|\s)+', Text),
+            (r'(\n|\s+)+', Whitespace),
             (r';.*?\n', Comment)
         ],
         'keyword': [
             # Regular keywords
             (words((
-                'acq_rel', 'acquire', 'add', 'addrspace', 'addrspacecast', 'afn', 'alias',
-                'aliasee', 'align', 'alignLog2', 'alignstack', 'alloca', 'allocsize', 'allOnes',
-                'alwaysinline', 'amdgpu_cs', 'amdgpu_es', 'amdgpu_gs', 'amdgpu_hs',
-                'amdgpu_kernel', 'amdgpu_ls', 'amdgpu_ps', 'amdgpu_vs', 'and', 'any',
-                'anyregcc', 'appending', 'arcp', 'argmemonly', 'args', 'arm_aapcs_vfpcc',
-                'arm_aapcscc', 'arm_apcscc', 'ashr', 'asm', 'atomic', 'atomicrmw', 'attributes',
-                'available_externally', 'avr_intrcc', 'avr_signalcc', 'bit', 'bitcast',
-                'bitMask', 'blockaddress', 'br', 'branchFunnel', 'builtin', 'byArg', 'byte',
-                'byteArray', 'byval', 'c', 'call', 'callee', 'caller', 'calls', 'catch',
-                'catchpad', 'catchret', 'catchswitch', 'cc', 'ccc', 'cleanup', 'cleanuppad',
-                'cleanupret', 'cmpxchg', 'cold', 'coldcc', 'comdat', 'common', 'constant',
-                'contract', 'convergent', 'critical', 'cxx_fast_tlscc', 'datalayout', 'declare',
-                'default', 'define', 'deplibs', 'dereferenceable', 'dereferenceable_or_null',
-                'distinct', 'dllexport', 'dllimport', 'dso_local', 'dso_preemptable',
-                'dsoLocal', 'eq', 'exact', 'exactmatch', 'extern_weak', 'external',
-                'externally_initialized', 'extractelement', 'extractvalue', 'fadd', 'false',
-                'fast', 'fastcc', 'fcmp', 'fdiv', 'fence', 'filter', 'flags', 'fmul',
-                'fpext', 'fptosi', 'fptoui', 'fptrunc', 'freeze', 'frem', 'from', 'fsub',
-                'funcFlags', 'function', 'gc', 'getelementptr', 'ghccc', 'global', 'guid', 'gv',
-                'hash', 'hhvm_ccc', 'hhvmcc', 'hidden', 'hot', 'hotness', 'icmp',
-                'ifunc', 'inaccessiblemem_or_argmemonly', 'inaccessiblememonly', 'inalloca',
-                'inbounds', 'indir', 'indirectbr', 'info', 'initialexec', 'inline',
-                'inlineBits', 'inlinehint', 'inrange', 'inreg', 'insertelement', 'insertvalue',
-                'insts', 'intel_ocl_bicc', 'inteldialect', 'internal', 'inttoptr', 'invoke',
-                'jumptable', 'kind', 'landingpad', 'largest', 'linkage', 'linkonce',
-                'linkonce_odr', 'live', 'load', 'local_unnamed_addr', 'localdynamic',
-                'localexec', 'lshr', 'max', 'metadata', 'min', 'minsize', 'module', 'monotonic',
-                'msp430_intrcc', 'mul', 'musttail', 'naked', 'name', 'nand', 'ne', 'nest',
-                'ninf', 'nnan', 'noalias', 'nobuiltin', 'nocapture', 'nocf_check',
-                'noduplicate', 'noduplicates', 'noimplicitfloat', 'noinline', 'none',
-                'nonlazybind', 'nonnull', 'norecurse', 'noRecurse', 'noredzone', 'noreturn',
-                'notail', 'notEligibleToImport', 'nounwind', 'nsw', 'nsz', 'null', 'nuw', 'oeq',
-                'offset', 'oge', 'ogt', 'ole', 'olt', 'one', 'opaque', 'optforfuzzing',
-                'optnone', 'optsize', 'or', 'ord', 'path', 'personality', 'phi', 'poison',
-                'prefix', 'preserve_allcc', 'preserve_mostcc', 'private', 'prologue',
-                'protected', 'ptrtoint', 'ptx_device', 'ptx_kernel', 'readnone', 'readNone',
-                'readonly', 'readOnly', 'reassoc', 'refs', 'relbf', 'release', 'resByArg',
-                'resume', 'ret', 'returnDoesNotAlias', 'returned', 'returns_twice', 'safestack',
-                'samesize', 'sanitize_address', 'sanitize_hwaddress', 'sanitize_memory',
-                'sanitize_thread', 'sdiv', 'section', 'select', 'seq_cst', 'sext', 'sge', 'sgt',
-                'shadowcallstack', 'shl', 'shufflevector', 'sideeffect', 'signext', 'single',
-                'singleImpl', 'singleImplName', 'sitofp', 'sizeM1', 'sizeM1BitWidth', 'sle',
-                'slt', 'source_filename', 'speculatable', 'spir_func', 'spir_kernel', 'srem',
-                'sret', 'ssp', 'sspreq', 'sspstrong', 'store', 'strictfp', 'sub', 'summaries',
-                'summary', 'swiftcc', 'swifterror', 'swiftself', 'switch', 'syncscope', 'tail',
-                'target', 'thread_local', 'to', 'token', 'triple', 'true', 'trunc', 'type',
-                'typeCheckedLoadConstVCalls', 'typeCheckedLoadVCalls', 'typeid', 'typeIdInfo',
-                'typeTestAssumeConstVCalls', 'typeTestAssumeVCalls', 'typeTestRes', 'typeTests',
-                'udiv', 'ueq', 'uge', 'ugt', 'uitofp', 'ule', 'ult', 'umax', 'umin', 'undef',
-                'une', 'uniformRetVal', 'uniqueRetVal', 'unknown', 'unnamed_addr', 'uno',
-                'unordered', 'unreachable', 'unsat', 'unwind', 'urem', 'uselistorder',
-                'uselistorder_bb', 'uwtable', 'va_arg', 'variable', 'vFuncId',
-                'virtualConstProp', 'void', 'volatile', 'weak', 'weak_odr', 'webkit_jscc',
+                'aarch64_sve_vector_pcs', 'aarch64_vector_pcs', 'acq_rel',
+                'acquire', 'add', 'addrspace', 'addrspacecast', 'afn', 'alias',
+                'aliasee', 'align', 'alignLog2', 'alignstack', 'alloca',
+                'allocsize', 'allOnes', 'alwaysinline', 'alwaysInline',
+                'amdgpu_cs', 'amdgpu_es', 'amdgpu_gfx', 'amdgpu_gs',
+                'amdgpu_hs', 'amdgpu_kernel', 'amdgpu_ls', 'amdgpu_ps',
+                'amdgpu_vs', 'and', 'any', 'anyregcc', 'appending', 'arcp',
+                'argmemonly', 'args', 'arm_aapcs_vfpcc', 'arm_aapcscc',
+                'arm_apcscc', 'ashr', 'asm', 'atomic', 'atomicrmw',
+                'attributes', 'available_externally', 'avr_intrcc',
+                'avr_signalcc', 'bit', 'bitcast', 'bitMask', 'blockaddress',
+                'blockcount', 'br', 'branchFunnel', 'builtin', 'byArg',
+                'byref', 'byte', 'byteArray', 'byval', 'c', 'call', 'callbr',
+                'callee', 'caller', 'calls', 'canAutoHide', 'catch',
+                'catchpad', 'catchret', 'catchswitch', 'cc', 'ccc',
+                'cfguard_checkcc', 'cleanup', 'cleanuppad', 'cleanupret',
+                'cmpxchg', 'cold', 'coldcc', 'comdat', 'common', 'constant',
+                'contract', 'convergent', 'critical', 'cxx_fast_tlscc',
+                'datalayout', 'declare', 'default', 'define', 'deplibs',
+                'dereferenceable', 'dereferenceable_or_null', 'distinct',
+                'dllexport', 'dllimport', 'dso_local', 'dso_local_equivalent',
+                'dso_preemptable', 'dsoLocal', 'eq', 'exact', 'exactmatch',
+                'extern_weak', 'external', 'externally_initialized',
+                'extractelement', 'extractvalue', 'fadd', 'false', 'fast',
+                'fastcc', 'fcmp', 'fdiv', 'fence', 'filter', 'flags', 'fmul',
+                'fneg', 'fpext', 'fptosi', 'fptoui', 'fptrunc', 'freeze',
+                'frem', 'from', 'fsub', 'funcFlags', 'function', 'gc',
+                'getelementptr', 'ghccc', 'global', 'guid', 'gv', 'hash',
+                'hhvm_ccc', 'hhvmcc', 'hidden', 'hot', 'hotness', 'icmp',
+                'ifunc', 'inaccessiblemem_or_argmemonly',
+                'inaccessiblememonly', 'inalloca', 'inbounds', 'indir',
+                'indirectbr', 'info', 'initialexec', 'inline', 'inlineBits',
+                'inlinehint', 'inrange', 'inreg', 'insertelement',
+                'insertvalue', 'insts', 'intel_ocl_bicc', 'inteldialect',
+                'internal', 'inttoptr', 'invoke', 'jumptable', 'kind',
+                'landingpad', 'largest', 'linkage', 'linkonce', 'linkonce_odr',
+                'live', 'load', 'local_unnamed_addr', 'localdynamic',
+                'localexec', 'lshr', 'max', 'metadata', 'min', 'minsize',
+                'module', 'monotonic', 'msp430_intrcc', 'mul', 'mustprogress',
+                'musttail', 'naked', 'name', 'nand', 'ne', 'nest', 'ninf',
+                'nnan', 'noalias', 'nobuiltin', 'nocallback', 'nocapture',
+                'nocf_check', 'noduplicate', 'noduplicates', 'nofree',
+                'noimplicitfloat', 'noinline', 'noInline', 'nomerge', 'none',
+                'nonlazybind', 'nonnull', 'noprofile', 'norecurse',
+                'noRecurse', 'noredzone', 'noreturn', 'nosync', 'notail',
+                'notEligibleToImport', 'noundef', 'nounwind', 'nsw',
+                'nsz', 'null', 'null_pointer_is_valid', 'nuw', 'oeq', 'offset',
+                'oge', 'ogt', 'ole', 'olt', 'one', 'opaque', 'optforfuzzing',
+                'optnone', 'optsize', 'or', 'ord', 'param', 'params',
+                'partition', 'path', 'personality', 'phi', 'poison',
+                'preallocated', 'prefix', 'preserve_allcc', 'preserve_mostcc',
+                'private', 'prologue', 'protected', 'ptrtoint', 'ptx_device',
+                'ptx_kernel', 'readnone', 'readNone', 'readonly', 'readOnly',
+                'reassoc', 'refs', 'relbf', 'release', 'resByArg', 'resume',
+                'ret', 'returnDoesNotAlias', 'returned', 'returns_twice',
+                'safestack', 'samesize', 'sanitize_address',
+                'sanitize_hwaddress', 'sanitize_memory', 'sanitize_memtag',
+                'sanitize_thread', 'sdiv', 'section', 'select', 'seq_cst',
+                'sext', 'sge', 'sgt', 'shadowcallstack', 'shl',
+                'shufflevector', 'sideeffect', 'signext', 'single',
+                'singleImpl', 'singleImplName', 'sitofp', 'sizeM1',
+                'sizeM1BitWidth', 'sle', 'slt', 'source_filename',
+                'speculatable', 'speculative_load_hardening', 'spir_func',
+                'spir_kernel', 'srem', 'sret', 'ssp', 'sspreq', 'sspstrong',
+                'store', 'strictfp', 'sub', 'summaries', 'summary', 'swiftcc',
+                'swifterror', 'swiftself', 'switch', 'syncscope', 'tail',
+                'tailcc', 'target', 'thread_local', 'to', 'token', 'triple',
+                'true', 'trunc', 'type', 'typeCheckedLoadConstVCalls',
+                'typeCheckedLoadVCalls', 'typeid', 'typeidCompatibleVTable',
+                'typeIdInfo', 'typeTestAssumeConstVCalls',
+                'typeTestAssumeVCalls', 'typeTestRes', 'typeTests', 'udiv',
+                'ueq', 'uge', 'ugt', 'uitofp', 'ule', 'ult', 'umax', 'umin',
+                'undef', 'une', 'uniformRetVal', 'uniqueRetVal', 'unknown',
+                'unnamed_addr', 'uno', 'unordered', 'unreachable', 'unsat',
+                'unwind', 'urem', 'uselistorder', 'uselistorder_bb', 'uwtable',
+                'va_arg', 'varFlags', 'variable', 'vcall_visibility',
+                'vFuncId', 'virtFunc', 'virtualConstProp', 'void', 'volatile',
+                'vscale', 'vTableFuncs', 'weak', 'weak_odr', 'webkit_jscc',
                 'win64cc', 'within', 'wpdRes', 'wpdResolutions', 'writeonly',
-                'x86_64_sysvcc', 'x86_fastcallcc', 'x86_intrcc', 'x86_mmx',
-                'x86_regcallcc', 'x86_stdcallcc', 'x86_thiscallcc', 'x86_vectorcallcc', 'xchg',
-                'xor', 'zeroext', 'zeroinitializer', 'zext', 'immarg', 'willreturn'),
+                'x', 'x86_64_sysvcc', 'x86_fastcallcc', 'x86_intrcc',
+                'x86_mmx', 'x86_regcallcc', 'x86_stdcallcc', 'x86_thiscallcc',
+                'x86_vectorcallcc', 'xchg', 'xor', 'zeroext',
+                'zeroinitializer', 'zext', 'immarg', 'willreturn'),
                 suffix=r'\b'), Keyword),
 
             # Types
             (words(('void', 'half', 'bfloat', 'float', 'double', 'fp128',
-                    'x86_fp80', 'ppc_fp128', 'label', 'metadata', 'token')),
+                    'x86_fp80', 'ppc_fp128', 'label', 'metadata', 'x86_mmx',
+                    'x86_amx', 'token')),
                    Keyword.Type),
 
             # Integer types
@@ -528,13 +559,13 @@ class LlvmMirBodyLexer(RegexLexer):
             # Flags
             (words(('killed', 'implicit')), Keyword),
             # ConstantInt values
-            (r'i[0-9]+ +', Keyword.Type, 'constantint'),
+            (r'(i[0-9]+)( +)', bygroups(Keyword.Type, Whitespace), 'constantint'),
             # ConstantFloat values
             (r'(half|float|double) +', Keyword.Type, 'constantfloat'),
             # Bare immediates
             include('integer'),
             # MMO's
-            (r':: *', Operator, 'mmo'),
+            (r'(::)( *)', bygroups(Operator, Whitespace), 'mmo'),
             # MIR Comments
             (r';.*', Comment),
             # If we get here, assume it's a target instruction
@@ -554,29 +585,29 @@ class LlvmMirBodyLexer(RegexLexer):
         ],
         'vreg': [
             # The bank or class if there is one
-            (r' *:(?!:)', Keyword, ('#pop', 'vreg_bank_or_class')),
+            (r'( *)(:(?!:))', bygroups(Whitespace, Keyword), ('#pop', 'vreg_bank_or_class')),
             # The LLT if there is one
-            (r' *\(', Text, 'vreg_type'),
+            (r'( *)(\()', bygroups(Whitespace, Text), 'vreg_type'),
             (r'(?=.)', Text, '#pop'),
         ],
         'vreg_bank_or_class': [
             # The unassigned bank/class
-            (r' *_', Name.Variable.Magic),
-            (r' *[a-zA-Z0-9_]+', Name.Variable),
+            (r'( *)(_)', bygroups(Whitespace, Name.Variable.Magic)),
+            (r'( *)([a-zA-Z0-9_]+)', bygroups(Whitespace, Name.Variable)),
             # The LLT if there is one
-            (r' *\(', Text, 'vreg_type'),
+            (r'( *)(\()', bygroups(Whitespace, Text), 'vreg_type'),
             (r'(?=.)', Text, '#pop'),
         ],
         'vreg_type': [
             # Scalar and pointer types
-            (r' *[sp][0-9]+', Keyword.Type),
-            (r' *<[0-9]+ *x *[sp][0-9]+>', Keyword.Type),
+            (r'( *)([sp][0-9]+)', bygroups(Whitespace, Keyword.Type)),
+            (r'( *)(<[0-9]+ *x *[sp][0-9]+>)', bygroups(Whitespace, Keyword.Type)),
             (r'\)', Text, '#pop'),
             (r'(?=.)', Text, '#pop'),
         ],
         'mmo': [
             (r'\(', Text),
-            (r' +', Text),
+            (r' +', Whitespace),
             (words(('load', 'store', 'on', 'into', 'from', 'align', 'monotonic',
                     'acquire', 'release', 'acq_rel', 'seq_cst')),
              Keyword),
@@ -650,7 +681,7 @@ class LlvmMirLexer(RegexLexer):
             (r'body: *\|', Keyword, 'llvm_mir_body'),
             # Consume everything else
             (r'.+', Text),
-            (r'\n', Text),
+            (r'\n', Whitespace),
         ],
         'name': [
             (r'[^\n]+', Name),
@@ -698,16 +729,16 @@ class NasmLexer(RegexLexer):
     floatn = decn + r'\.e?' + decn
     string = r'"(\\"|[^"\n])*"|' + r"'(\\'|[^'\n])*'|" + r"`(\\`|[^`\n])*`"
     declkw = r'(?:res|d)[bwdqt]|times'
-    register = (r'r[0-9][0-5]?[bwd]?|'
+    register = (r'(r[0-9][0-5]?[bwd]?|'
                 r'[a-d][lh]|[er]?[a-d]x|[er]?[sb]p|[er]?[sd]i|[c-gs]s|st[0-7]|'
-                r'mm[0-7]|cr[0-4]|dr[0-367]|tr[3-7]')
+                r'mm[0-7]|cr[0-4]|dr[0-367]|tr[3-7])\b')
     wordop = r'seg|wrt|strict'
     type = r'byte|[dq]?word'
     # Directives must be followed by whitespace, otherwise CPU will match
     # cpuid for instance.
     directives = (r'(?:BITS|USE16|USE32|SECTION|SEGMENT|ABSOLUTE|EXTERN|GLOBAL|'
                   r'ORG|ALIGN|STRUC|ENDSTRUC|COMMON|CPU|GROUP|UPPERCASE|IMPORT|'
-                  r'EXPORT|LIBRARY|MODULE)\s+')
+                  r'EXPORT|LIBRARY|MODULE)(?=\s)')
 
     flags = re.IGNORECASE | re.MULTILINE
     tokens = {
@@ -716,12 +747,12 @@ class NasmLexer(RegexLexer):
             include('whitespace'),
             (identifier + ':', Name.Label),
             (r'(%s)(\s+)(equ)' % identifier,
-                bygroups(Name.Constant, Keyword.Declaration, Keyword.Declaration),
+                bygroups(Name.Constant, Whitespace, Keyword.Declaration),
                 'instruction-args'),
             (directives, Keyword, 'instruction-args'),
             (declkw, Keyword.Declaration, 'instruction-args'),
             (identifier, Name.Function, 'instruction-args'),
-            (r'[\r\n]+', Text)
+            (r'[\r\n]+', Whitespace)
         ],
         'instruction-args': [
             (string, String),
@@ -733,7 +764,7 @@ class NasmLexer(RegexLexer):
             include('punctuation'),
             (register, Name.Builtin),
             (identifier, Name.Variable),
-            (r'[\r\n]+', Text, '#pop'),
+            (r'[\r\n]+', Whitespace, '#pop'),
             include('whitespace')
         ],
         'preproc': [
@@ -742,9 +773,10 @@ class NasmLexer(RegexLexer):
             (r'\n', Comment.Preproc, '#pop'),
         ],
         'whitespace': [
-            (r'\n', Text),
-            (r'[ \t]+', Text),
-            (r';.*', Comment.Single)
+            (r'\n', Whitespace),
+            (r'[ \t]+', Whitespace),
+            (r';.*', Comment.Single),
+            (r'#.*', Comment.Single)
         ],
         'punctuation': [
             (r'[,():\[\]]+', Punctuation),
@@ -792,9 +824,9 @@ class TasmLexer(RegexLexer):
     floatn = decn + r'\.e?' + decn
     string = r'"(\\"|[^"\n])*"|' + r"'(\\'|[^'\n])*'|" + r"`(\\`|[^`\n])*`"
     declkw = r'(?:res|d)[bwdqt]|times'
-    register = (r'r[0-9][0-5]?[bwd]|'
+    register = (r'(r[0-9][0-5]?[bwd]|'
                 r'[a-d][lh]|[er]?[a-d]x|[er]?[sb]p|[er]?[sd]i|[c-gs]s|st[0-7]|'
-                r'mm[0-7]|cr[0-4]|dr[0-367]|tr[3-7]')
+                r'mm[0-7]|cr[0-4]|dr[0-367]|tr[3-7])\b')
     wordop = r'seg|wrt|strict'
     type = r'byte|[dq]?word'
     directives = (r'BITS|USE16|USE32|SECTION|SEGMENT|ABSOLUTE|EXTERN|GLOBAL|'
@@ -813,11 +845,11 @@ class TasmLexer(RegexLexer):
             (identifier + ':', Name.Label),
             (directives, Keyword, 'instruction-args'),
             (r'(%s)(\s+)(%s)' % (identifier, datatype),
-                bygroups(Name.Constant, Keyword.Declaration, Keyword.Declaration),
+                bygroups(Name.Constant, Whitespace, Keyword.Declaration),
                 'instruction-args'),
             (declkw, Keyword.Declaration, 'instruction-args'),
             (identifier, Name.Function, 'instruction-args'),
-            (r'[\r\n]+', Text)
+            (r'[\r\n]+', Whitespace)
         ],
         'instruction-args': [
             (string, String),
@@ -830,8 +862,8 @@ class TasmLexer(RegexLexer):
             (register, Name.Builtin),
             (identifier, Name.Variable),
             # Do not match newline when it's preceeded by a backslash
-            (r'(\\\s*)(;.*)([\r\n])', bygroups(Text, Comment.Single, Text)),
-            (r'[\r\n]+', Text, '#pop'),
+            (r'(\\)(\s*)(;.*)([\r\n])', bygroups(Text, Whitespace, Comment.Single, Whitespace)),
+            (r'[\r\n]+', Whitespace, '#pop'),
             include('whitespace')
         ],
         'preproc': [
@@ -840,9 +872,9 @@ class TasmLexer(RegexLexer):
             (r'\n', Comment.Preproc, '#pop'),
         ],
         'whitespace': [
-            (r'[\n\r]', Text),
-            (r'\\[\n\r]', Text),
-            (r'[ \t]+', Text),
+            (r'[\n\r]', Whitespace),
+            (r'(\\)([\n\r])', bygroups(Text, Whitespace)),
+            (r'[ \t]+', Whitespace),
             (r';.*', Comment.Single)
         ],
         'punctuation': [
@@ -875,7 +907,7 @@ class Ca65Lexer(RegexLexer):
     tokens = {
         'root': [
             (r';.*', Comment.Single),
-            (r'\s+', Text),
+            (r'\s+', Whitespace),
             (r'[a-z_.@$][\w.@$]*:', Name.Label),
             (r'((ld|st)[axy]|(in|de)[cxy]|asl|lsr|ro[lr]|adc|sbc|cmp|cp[xy]'
              r'|cl[cvdi]|se[cdi]|jmp|jsr|bne|beq|bpl|bmi|bvc|bvs|bcc|bcs'
@@ -956,7 +988,7 @@ class Dasm16Lexer(RegexLexer):
             (identifier + ':', Name.Label),
             (instruction, Name.Function, 'instruction-args'),
             (r'\.' + identifier, Name.Function, 'data-args'),
-            (r'[\r\n]+', Text)
+            (r'[\r\n]+', Whitespace)
         ],
 
         'numeric' : [
@@ -978,7 +1010,7 @@ class Dasm16Lexer(RegexLexer):
         ],
 
         'instruction-line' : [
-            (r'[\r\n]+', Text, '#pop'),
+            (r'[\r\n]+', Whitespace, '#pop'),
             (r';.*?$', Comment, '#pop'),
             include('whitespace')
         ],
@@ -998,8 +1030,8 @@ class Dasm16Lexer(RegexLexer):
         ],
 
         'whitespace': [
-            (r'\n', Text),
-            (r'\s+', Text),
+            (r'\n', Whitespace),
+            (r'\s+', Whitespace),
             (r';.*?\n', Comment)
         ],
     }
